#!/usr/bin/python
#
# Setup XenStore entry for paravirtualized USB driver.
# 
# By Marek Marczykowski <marmarek@invisiblethinglab.com>
# based on bash script by Noboru Iwamatsu <n_iwamatsu@xxxxxxxxxxxxxx>
#

import xen.lowlevel.xs
import sys

xs = xen.lowlevel.xs.xs()


dev_name = "vusb"
# Max 16 ports.
num_ports = 8

def usage ():
    print "Usage: initpv.py <backend-id> <frontend-id> <device-id>"
    print "    <backend-id>: the domain id of backend"
    print "    <frontend-id>: the domain id of frontend"
    print "    <device-id>: the device id of frontend"
    print ""
    print "Example:"
    print "    If you use paravirtual USB driver on Domain ID 1,"
    print "    simply do"
    print "    initpv.py 0 1 0"
    sys.exit(1)


def setup(backend, frontend, devid):

	trans = xs.transaction_start()

	be_path = "/local/domain/%d/backend/%s/%d/%d" % (backend, dev_name, frontend, devid)
	fe_path = "/local/domain/%d/device/%s/%d" % (frontend, dev_name, devid)

	be_perm = [{'dom': backend}, {'dom': frontend, 'read': True} ]
	fe_perm = [{'dom': frontend}, {'dom': backend, 'read': True} ]

	# Create directories and set permissions
	xs.write(trans, be_path, "")
	xs.set_permissions(trans, be_path, be_perm)

	xs.write(trans, fe_path, "")
	xs.set_permissions(trans, fe_path, fe_perm)

	# Write backend information into the location that frontend look for.
	xs.write(trans, "%s/backend-id" % fe_path, str(backend))
	xs.write(trans, "%s/backend" % fe_path, be_path)

	# Write frontend information into the location that backend look for.
	xs.write(trans, "%s/frontend-id" % be_path, str(frontend))
	xs.write(trans, "%s/frontend" % be_path, fe_path)

	# Write USB Spec version field.
	xs.write(trans, "%s/usb-ver" % be_path, "2")

	# Write virtual root hub field.
	xs.write(trans, "%s/num-ports" % be_path, str(num_ports))
	for port in range(1, num_ports+1):
		# Set all port to disconnected state
		xs.write(trans, "%s/port/%d" % (be_path, port), "")

	# Set state to XenbusStateInitialising
	xs.write(trans, "%s/state" % fe_path, "1")
	xs.write(trans, "%s/state" % be_path, "1")
	xs.write(trans, "%s/online" % be_path, "1")

	xs.transaction_end(trans)



def main():
	if len(sys.argv) < 3:
		usage()

	setup(int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3]))

main()
